@@ -1,6 +1,6 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (c) 2008  LiTL, LLC
+ * Copyright (c) 2008  litl, LLC
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -32,8 +32,7 @@
 
 G_BEGIN_DECLS
 
-JSObject* gjs_define_ns(JSContext    *context,
-                        JSObject     *in_object,
+JSObject* gjs_create_ns(JSContext    *context,
                         const char   *ns_name,
                         GIRepository *repo);
 
