@@ -26,9 +26,8 @@
 #include <util/log.h>
 #include <util/glib.h>
 
-#include <jsapi.h>
-
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/importer.h>
 #include <gjs/compat.h>
 
 #include <string.h>
@@ -367,13 +366,14 @@ load_module_elements(JSContext *context,
         }
 
         while (!JSID_IS_VOID(idp)) {
-            const char *name;
+            char *name;
 
             if (!gjs_get_string_id(context, idp, &name)) {
                 continue;
             }
 
-            g_ptr_array_add(iter->elements, g_strdup(name));
+            /* Pass ownership of name */
+            g_ptr_array_add(iter->elements, name);
 
             if (!JS_NextProperty(context, jsiter, &idp)) {
                 break;
@@ -514,6 +514,15 @@ do_import(JSContext  *context,
     full_path = NULL;
     directories = NULL;
 
+    /* First try importing an internal module like byteArray */
+    if (gjs_is_registered_native_module(context, obj, name) &&
+        import_native_file(context, obj, name, NULL)) {
+        gjs_debug(GJS_DEBUG_IMPORTER,
+                  "successfully imported module '%s'", name);
+        result = JS_TRUE;
+        goto out;
+    }
+
     for (i = 0; i < search_path_len; ++i) {
         jsval elem;
 
@@ -539,6 +548,10 @@ do_import(JSContext  *context,
         if (!gjs_string_to_utf8(context, elem, &dirname))
             goto out; /* Error message already set */
 
+        /* Ignore empty path elements */
+        if (dirname[0] == '\0')
+            continue;
+
         /* Try importing __init__.js and loading the symbol from it */
         if (full_path)
             g_free(full_path);
@@ -564,14 +577,6 @@ do_import(JSContext  *context,
             }
         }
 
-        /* First try importing an internal module like byteArray */
-        if (import_native_file(context, obj, name, NULL)) {
-            gjs_debug(GJS_DEBUG_IMPORTER,
-                      "successfully imported module '%s'", name);
-            result = JS_TRUE;
-            goto out;
-        }
-
         /* Second try importing a directory (a sub-importer) */
         if (full_path)
             g_free(full_path);
@@ -729,7 +734,7 @@ importer_new_enumerate(JSContext  *context,
     ImporterIterator *iter;
 
     switch (enum_op) {
-#ifdef HAVE_JSENUMERATE_INIT_ALL
+#ifdef HAVE_MOZJS_2
     case JSENUMERATE_INIT_ALL:
 #endif
     case JSENUMERATE_INIT: {
@@ -919,7 +924,8 @@ importer_new_resolve(JSContext *context,
                      JSObject **objp)
 {
     Importer *priv;
-    const char *name;
+    char *name;
+    JSBool ret = JS_TRUE;
 
     *objp = NULL;
 
@@ -930,23 +936,25 @@ importer_new_resolve(JSContext *context,
     if (strcmp(name, "valueOf") == 0 ||
         strcmp(name, "toString") == 0 ||
         strcmp(name, "__iterator__") == 0)
-        return JS_TRUE;
+        goto out;
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_IMPORTER, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_TRUE; /* we are the prototype, or have the wrong class */
+    if (priv == NULL) /* we are the prototype, or have the wrong class */
+        goto out;
 
     JS_BeginRequest(context);
     if (do_import(context, obj, priv, name)) {
         *objp = obj;
-        JS_EndRequest(context);
-        return JS_TRUE;
     } else {
-        JS_EndRequest(context);
-        return JS_FALSE;
+        ret = JS_FALSE;
     }
+    JS_EndRequest(context);
+
+ out:
+    g_free(name);
+    return ret;
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
@@ -957,24 +965,25 @@ importer_new_resolve(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
-static JSBool
-importer_constructor(JSContext *context,
-                     JSObject  *obj,
-                     uintN      argc,
-                     jsval     *argv,
-                     jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(importer)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(importer)
     Importer *priv;
 
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(importer);
+    (void) argv;
+
     priv = g_slice_new0(Importer);
 
     GJS_INC_COUNTER(importer);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
-                        "importer constructor, obj %p priv %p", obj, priv);
+                        "importer constructor, obj %p priv %p", object, priv);
+
+    GJS_NATIVE_CONSTRUCTOR_FINISH(importer);
 
     return JS_TRUE;
 }
@@ -1013,7 +1022,7 @@ static struct JSClass gjs_importer_class = {
     JS_PropertyStub,
     JS_PropertyStub,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_StrictPropertyStub,
     (JSEnumerateOp) importer_new_enumerate, /* needs cast since it's the new enumerate signature */
     (JSResolveOp) importer_new_resolve, /* needs cast since it's the new resolve signature */
     JS_ConvertStub,
@@ -1035,6 +1044,7 @@ importer_new(JSContext    *context)
     JSObject *importer;
     Importer *priv;
     JSObject *global;
+    (void) priv;
 
     global = gjs_get_import_global(context);
 
@@ -1051,7 +1061,7 @@ importer_new(JSContext    *context)
                                   * none - just name the prototype like
                                   * Math - rarely correct)
                                   */
-                                 importer_constructor,
+                                 gjs_importer_constructor,
                                  /* number of constructor args */
                                  0,
                                  /* props of prototype */
@@ -1075,8 +1085,6 @@ importer_new(JSContext    *context)
     if (importer == NULL)
         gjs_fatal("No memory to create ns object");
 
-    priv = priv_from_js(context, importer);
-
     return importer;
 }
 
